from pydantic import BaseModel, Field, field_validator
from dhuolib.messages import (
    VALUE_NULL_OR_EMPTY,
    MODEL_STAGE_TYPE,
    EXPERIMENT_FIELD_DESCRIPTION,
)


class BaseModelWithConfig(BaseModel):
    class Config:
        protected_namespaces = ()


class ModelBody(BaseModelWithConfig):
    modelname: str = Field(..., description="Model Name")
    tags: dict = Field(None, description="Tags")
    stage: str = Field(..., description=MODEL_STAGE_TYPE)
    run_id: str = Field(..., description="Run ID")
    model_uri: str = Field(..., description="Model URI")

    @field_validator("modelname", "stage", "run_id", "model_uri")
    def validate_empty_values(cls, value):
        if not value:
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class RunExperiment(BaseModelWithConfig):
    type_model: str = Field(
        ..., description="RANDOM_FOREST|XGBOOST|LINEAR_REGRESSION|LOGISTIC_REGRESSION"
    )
    experiment_name: str = Field(..., description=EXPERIMENT_FIELD_DESCRIPTION)
    tags: dict = Field(None, description="Tags")

    @field_validator("type_model", "experiment_name")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class RunFiles(BaseModelWithConfig):
    modelpkl_path: str = Field(..., description="Path to model.pkl")
    requirements_path: str = Field(..., description="Path to requirements.txt")

    @field_validator("modelpkl_path", "requirements_path")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class ExperimentBody(BaseModelWithConfig):
    experiment_name: str = Field(..., description="Id")
    experiment_tags: dict = Field(None, description="Tags")


class PredictOnlineModelBody(BaseModelWithConfig):
    run_id: str = Field(None, description="Run ID")
    modelname: str = Field(..., description="DEPENDENCY|PREDICT")
    stage: str = Field(..., description=MODEL_STAGE_TYPE)

    @field_validator("modelname", "stage")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class PredictBatchModelBody(BaseModelWithConfig):
    run_id: str = Field(None, description="Run ID")
    modelname: str = Field(..., description="DEPENDENCY|PREDICT")
    stage: str = Field(None, description=MODEL_STAGE_TYPE)
    batch_model_dir: str = Field(..., description="Batch File")
    experiment_name: str = Field(..., description=EXPERIMENT_FIELD_DESCRIPTION)
    type_model: str = Field(
        ..., description="RANDOM_FOREST|XGBOOST|LINEAR_REGRESSION|LOGISTIC_REGRESSION"
    )

    @field_validator("modelname", "batch_model_dir", "experiment_name", "type_model")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class ConnectModel(BaseModelWithConfig):
    dialect_drive: str = Field(..., description="Dialect")
    username: str = Field(..., description="Username")
    password: str = Field(..., description="Password")
    host: str = Field(..., description="Host")
    port: str = Field(..., description="Port")
    service_name: str = Field(..., description="Service Name")

    @field_validator(
        "dialect_drive", "username", "password", "host", "port", "service_name"
    )
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class TransitionModel(BaseModelWithConfig):
    model_name: str = Field(..., description="Model Name")
    version: str = Field(..., description="Version")
    stage: str = Field(..., description=MODEL_STAGE_TYPE)

    @field_validator("model_name", "stage")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value


class RunSearch(BaseModelWithConfig):
    filter_string: str = Field(None, description="Filter String")
    max_results: int = Field(None, description="Max Results")
    page_token: str = Field(None, description="Page Token")
    experiment_name: str = Field(..., description=EXPERIMENT_FIELD_DESCRIPTION)

    @field_validator("filter_string", "experiment_name")
    def validate_empty_values(cls, value):
        if value == "":
            raise ValueError(VALUE_NULL_OR_EMPTY)
        return value



import base64
import requests

from dhuolib.config import logger
from dhuolib.services import ServiceAPIMLFacade
from dhuolib.utils import validate_name


class DhuolibPlatformClient:
    def __init__(self, service_uri=None, project_name=None, token=None):
        if not service_uri:
            raise ValueError("service_uri is required")
        if not token:
            raise ValueError("token is required")

        self.service = ServiceAPIMLFacade(service_uri, token=token)
        self.project_name = project_name

    def create_batch_project(self, project_name: str):
        if project_name:
            project_name = validate_name(project_name)
        else:
            project_name = validate_name(self.project_name)

        try:
            response = self.service.project_api.create_project(project_name)
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        if response.status_code == 400:
            raise ValueError("Project already exists")
        elif response.status_code == 404:
            return ConnectionError("Connection error")

        return response.json()

    def deploy_batch_project(
        self,
        project_name: str = "",
        script_filename: str = "",
        requirements_filename: str = "",
    ):

        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")

        if not (script_filename or requirements_filename):
            raise ValueError("script_filename and requirements_filename are required")

        try:
            with open(script_filename, "rb") as script_file, open(
                requirements_filename, "rb"
            ) as requirements_file:
                encoded_script = base64.b64encode(script_file.read())
                encoded_requirements = base64.b64encode(requirements_file.read())
                try:
                    response = self.service.project_api.deploy_script(
                        project_name=project_name,
                        script_file_encode=encoded_script,
                        requirements_file_enconde=encoded_requirements,
                    )
                except requests.exceptions.HTTPError as e:
                    logger.error(f"Error: {e}")
                    return {"error": str(e)}
                return response
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def pipeline_status_report(self, project_name: str = ""):
        lst = []

        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")
        response = self.service.project_api.get_pipeline_status(project_name)

        if "data" in response.keys():
            for data in response["data"]:
                lst.append(
                    {
                        "date_log": data["date_log"],
                        "step": data["step"],
                        "status": data["status"],
                    }
                )
            return lst

        return response

    def create_cluster(self, project_name: str = "", cluster_size: int = 1):

        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")

        if cluster_size not in [1, 2, 3]:
            raise ValueError("cluster_size must be 1, 2 or 3")

        try:
            response = self.service.project_api.create_cluster(
                project_name, cluster_size
            )
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        return response

    def batch_run(self, project_name: str = ""):
        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")

        try:
            response = self.service.project_api.run_pipeline(project_name)
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        return response

    def schedule_batch_run(self, project_name: str = "", schedule_interval: str = ""):
        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")

        try:
            response = self.service.project_api.create_schedule(
                project_name, schedule_interval
            )
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        return response

    def remove_schedule(self, project_name: str = ""):
        if not project_name:
            project_name = self.project_name

        if not project_name:
            raise ValueError("Batch project is required")

        try:
            response = self.service.project_api.remove_schedule(project_name)
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        return response



import sys
import unittest
from unittest.mock import mock_open, patch

sys.path.append("src")

from dhuolib.clients.experiment import DhuolibExperimentMLClient


class TestDhuolibPlatformClient(unittest.TestCase):
    def setUp(self):
        self.end_point = "http://localhost:8000"
        self.dhuolib = DhuolibExperimentMLClient(
            service_uri=self.end_point, token="123"
        )

    @patch("requests.post")
    def test_1_run_experiment_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "experiment_name": "experiment_name",
            "run_id": "run_id",
            "model_uri": "model_uri",
        }

        response = self.dhuolib.execute_run_for_experiment(
            type_model="teste1",
            experiment_name="2",
            tags={"version": "v1", "priority": "P1"},
            modelpkl_path="tests/files/LogisticRegression_best.pickle",
            requirements_path="tests/files/requirements.txt",
        )

        self.assertEqual(response, mock_response.json.return_value)



import json
import requests
from abc import ABC, abstractmethod


class APIError(Exception):
    pass


class ServiceAPI(ABC):
    def _handle_response(self, response):
        if not response.ok:
            return {
                "error": f"API request failed with status {response.status_code}: {response.text}"
            }
        return response.json()


class ServiceML(ServiceAPI):
    def __init__(self, service_uri: str = "http://localhost:8000", token: str = None):
        self.service_uri = f"{service_uri}/api"
        self.token = token

    @abstractmethod
    def create(self, params, **kwargs):
        pass

    @abstractmethod
    def search(
        self,
        filter_string: str = "",
        max_results: int = 10,
        page_token: str = None,
        **kwargs,
    ):
        pass


class ExperimentMLAPI(ServiceML):
    def __init__(self, service_uri, token):
        super().__init__(service_uri, token)
        self.service_uri = f"{self.service_uri}/experiment"
        if "Bearer" in token:
            self.authorization_header = {"Authorization": f"{token}"}
        else:
            self.authorization_header = {"X-Dhuo-Access-Key": f"{token}"}

        self.headers = {"Content-Type": "application/json", **self.authorization_header}

    def create(self, experiment_params: dict):
        response = requests.post(
            f"{self.service_uri}",
            data=json.dumps(experiment_params),
            headers=self.headers,
        )
        return self._handle_response(response)

    def search(
        self,
        filter_string: str = "",
        max_results: int = 10,
        page_token: str = "",
        view_type: int = 1,
    ):
        response = requests.get(
            f"{self.service_uri}/search?filter_string={filter_string}&max_results={max_results}&page_token={page_token}&view_type={view_type}",
            headers=self.authorization_header,
        )
        return self._handle_response(response)

    def predict_online(self, params={}, files=None):
        if params is None and not isinstance(params, dict):
            raise ValueError("json_data must be a dict")
        response = requests.post(
            f"{self.service_uri}/predict_online",
            data=params,
            files=files,
        )
        return self._handle_response(response)

    def download_pickle(
        self,
        experiment_name: str,
        type_model: str,
        model_name: str,
        model_stage: str = "",
        run_id: str = "",
        local_filename: str = "model.pickle",
    ):
        url = f"{self.service_uri}/download/batch/{experiment_name}/{model_name}?model_stage={model_stage}&run_id={run_id}&type_model={type_model}"
        with requests.get(url, stream=True, headers=self.authorization_header) as r:
            r.raise_for_status()
            with open(local_filename, "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        return local_filename


class ModelMLAPI(ServiceML):
    def __init__(self, service_uri, token):
        super().__init__(service_uri, token)
        self.service_uri = f"{self.service_uri}/models"
        if "Bearer" in token:
            self.authorization_header = {"Authorization": f"{token}"}
        else:
            self.authorization_header = {"X-Dhuo-Access-Key": f"{token}"}

        self.headers = {"Content-Type": "application/json", **self.authorization_header}

    def create(self, model_params):
        if model_params is None and not isinstance(model_params, dict):
            raise ValueError("json_data must be a dict")
        response = requests.post(
            f"{self.service_uri}",
            data=json.dumps(model_params),
            headers=self.headers,
        )
        return self._handle_response(response)

    def transition_model(self, model_name: str, version: str, stage: str):
        response = requests.get(
            f"{self.service_uri}/transition-model/{model_name}?version={version}&stage={stage}",
            headers=self.authorization_header,
        )
        return self._handle_response(response)

    def search(
        self, filter_string: str = "", max_results: int = 10, page_token: str = ""
    ):
        response = requests.get(
            f"{self.service_uri}/search?filter_string={filter_string}&max_results={max_results}&page_token={page_token}",
            headers=self.authorization_header,
        )
        return self._handle_response(response)


class RunsMLAPI(ServiceML):
    def __init__(self, service_uri, token):
        super().__init__(service_uri, token)
        self.service_uri = f"{self.service_uri}/runs"
        if "Bearer" in token:
            self.authorization_header = {"Authorization": f"{token}"}
        else:
            self.authorization_header = {"X-Dhuo-Access-Key": f"{token}"}

    def create(self, run_params):
        response = requests.post(
            f"{self.service_uri}",
            json=run_params,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )
        return self._handle_response(response)

    def upload_chunk(
        self,
        experiment_name: str,
        file_name: str,
        file_path: str,
        chunk: bytes,
        content_type: str,
    ):
        response = requests.post(
            f"{self.service_uri}/upload_chunk",
            data={"experiment_name": experiment_name, "file_name": file_name},
            files={file_name: (file_path, chunk, content_type)},
            headers=self.authorization_header,
        )
        if response.status_code != 200:
            return {"error": "Failed to upload chunk"}

    def search(
        self,
        filter_string: str = "",
        max_results: int = 10,
        page_token: str = "",
        experiment_name: str = "",
    ):
        response = requests.get(
            f"{self.service_uri}/{experiment_name}?filter_string={filter_string}&max_results={max_results}&page_token={page_token}",
            headers=self.authorization_header,
        )
        if response.status_code == 204:
            raise ValueError("Experiment not found")
        return self._handle_response(response)


class ProjectMLAPI(ServiceAPI):
    def __init__(self, service_uri, token):

        if not isinstance(service_uri, str):
            raise ValueError("service_uri must be a string")

        self.service_uri = f"{service_uri}/api"
        if "Bearer" in token:
            self.authorization_header = {"Authorization": f"{token}"}
        else:
            self.authorization_header = {"X-Dhuo-Access-Key": f"{token}"}

    def create_project(self, project_name):
        body = {"project_name": project_name}
        return requests.post(
            f"{self.service_uri}/project",
            json=body,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )

    def deploy_script(
        self, project_name: str, script_file_encode: str, requirements_file_enconde: str
    ):
        body = {
            "project_name": project_name,
            "requirements_content": requirements_file_enconde.decode("utf-8"),
            "run_script_content": script_file_encode.decode("utf-8"),
        }
        response = requests.post(
            f"{self.service_uri}/deploy",
            json=body,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )

        return self._handle_response(response)

    def get_pipeline_status(self, project_name: str):
        route = "deploy/{}".format(project_name)
        response = requests.get(
            f"{self.service_uri}/{route}", headers=self.authorization_header
        )

        return self._handle_response(response)

    def create_cluster(self, project_name: str, cluster_size: int):
        body = {"project_name": project_name, "cluster_size": cluster_size}
        response = requests.post(
            f"{self.service_uri}/cluster",
            json=body,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )

        return self._handle_response(response)

    def run_pipeline(self, project_name: str):
        body = {"project_name": project_name}
        response = requests.post(
            f"{self.service_uri}/cluster/run",
            json=body,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )

        return self._handle_response(response)

    def remove_schedule(self, project_name: str):
        response = requests.delete(
            f"{self.service_uri}/cluster/schedule/{project_name}",
            headers=self.authorization_header,
        )

        return self._handle_response(response)

    def create_schedule(self, project_name: str, schedule_interval: str):
        body = {"project_name": project_name, "schedule_interval": schedule_interval}
        response = requests.post(
            f"{self.service_uri}/cluster/schedule",
            json=body,
            headers={"Content-Type": "application/json", **self.authorization_header},
        )

        return self._handle_response(response)


class ServiceAPIMLFacade:
    def __init__(self, service_uri, token):
        self.experiment_api = ExperimentMLAPI(service_uri, token)
        self.model_api = ModelMLAPI(service_uri, token)
        self.runs_api = RunsMLAPI(service_uri, token)
        self.project_api = ProjectMLAPI(service_uri, token)



import unittest
import sys

sys.path.append("src")

import dhuolib.utils as utils


class TestDhuolibUtils(unittest.TestCase):
    def test_0_transform_name_project_no_project_name(self):
        with self.assertRaises(ValueError):
            project_name = utils.validate_name(None)
            self.assertIsNone(project_name)

    def test_1_project_name_min_length(self):
        with self.assertRaises(ValueError) as context:
            utils.validate_name("abc")
            self.assertTrue(
                "project_name must have at least 4 characters" in str(context.exception)
            )

    def test_2_transform_name_project(self):
        with self.assertRaises(ValueError):
            utils.validate_name("Projeto Analise Sentimento")

    def test_3_transform_name_project(self):
        project_name = utils.validate_name("Projeto_Analise_Sentimento")
        self.assertEqual(project_name, "projeto_analise_sentimento")



import json
import sys
import pandas as pd

from contextlib import contextmanager
from datetime import datetime
from pathlib import Path
from sqlalchemy import create_engine, inspect, Float, Integer, text
from sqlalchemy.types import VARCHAR
from sqlalchemy.dialects.oracle import FLOAT as OracleFLOAT
from sqlalchemy.orm import scoped_session, sessionmaker

from dhuolib.config import logger


class DatabaseConnection:
    def __init__(self, config_file_name=None):
        if self.in_dataflow():
            self.connection_string = f"oracle+oracledb://{sys.argv[1]}"
        else:
            f = open(config_file_name)
            data = json.load(f)
            self.connection_string = data["connection_string"]

        self.engine = self._get_engine(self.connection_string)
        self.session = scoped_session(sessionmaker(bind=self.engine))

    def in_dataflow(self):
        if str(Path.home()) == "/home/dataflow":
            return True
        return False

    def _get_engine(self, connection_string):
        self.engine = create_engine(connection_string)
        return self.engine

    @contextmanager
    def session_scope(self, expire=False):
        self.session.expire_on_commit = expire
        try:
            yield self.session
            logger.info(f"Sessão foi iniciada {self.session}")
            self.session.commit()
        except Exception as e:
            self.session.rollback()
            logger.error(f"Erro na sessão {self.session}: {e}")
            raise
        finally:
            self.session.close()
            logger.info(f"Sessão foi finalizada {self.session}")


class GenericRepository:
    def __init__(self, db_connection):
        self.db = db_connection

    def map_dtype(self, df, engine):
        dtype_mapping = {}
        for col, dtype in df.dtypes.items():
            if pd.api.types.is_integer_dtype(dtype):
                dtype_mapping[col] = Integer()
            elif pd.api.types.is_float_dtype(dtype):
                if engine.dialect.name == "oracle":
                    dtype_mapping[col] = Float(precision=53).with_variant(
                        OracleFLOAT(binary_precision=53), "oracle"
                    )
                else:
                    dtype_mapping[col] = Float()
            elif dtype == "object":
                dtype_mapping[col] = VARCHAR(255)
            else:
                dtype_mapping[col] = VARCHAR(255)
        return dtype_mapping

    def create_table_by_dataframe(self, table_name: str, df: pd.DataFrame):
        if not table_name or df.empty:
            raise ValueError("table_name or data is required")

        with self.db.session_scope() as session:
            inspector = inspect(session.bind)
            if table_name in inspector.get_table_names():
                return f"A tabela {table_name} já existe. Utilize o método update_table_by_dataframe para atualizar a tabela."

            dtype_mapping = self.map_dtype(df, session.bind)

            num_rows_inserted = df.to_sql(
                name=table_name,
                con=session.bind,
                if_exists="replace",
                index=False,
                dtype=dtype_mapping,
            )

        return {"number_lines_inserted": num_rows_inserted, "table_name": table_name}

    # replace_or_append tabela na base de dados
    def update_table_by_dataframe(
        self,
        table_name: str,
        dataframe: pd.DataFrame,
        if_exists: str = "append",
        is_update_version: bool = False,
        force_replace_delete_itens_in_table: bool = False,
    ):
        if if_exists not in ["replace", "append"]:
            raise ValueError("if_exists must be 'append' or 'replace'")

        if not force_replace_delete_itens_in_table and if_exists == "replace":
            confirmation = input(
                "Você tem certeza que deseja fazer a substituição de todos os dados da tabela? Essa operação uma vez escolhida não pode ser desfeita (yes/no): "
            )
            if confirmation.lower() != "yes":
                print("Operação cancelada.")
                return

        if not table_name or dataframe.empty:
            raise ValueError("table_name, dataframe are required")
        with self.db.session_scope() as session:
            if is_update_version:
                df = pd.read_sql(
                    f"SELECT version FROM {table_name.lower()}", con=session.bind
                )
                latest_version = df["version"].max()
                if pd.isna(latest_version):
                    latest_version = 1
                else:
                    latest_version = int(latest_version) + 1

                dataframe["version"] = latest_version
                dataframe["created_at"] = datetime.now()

                dataframe["created_at"] = pd.to_datetime(dataframe["created_at"])

            if if_exists == "replace":
                session.execute(text(f"DELETE FROM {table_name}"))
                dataframe.to_sql(
                    name=table_name, con=session.bind, if_exists="append", index=False
                )
            else:
                dataframe.to_sql(
                    name=table_name, con=session.bind, if_exists="append", index=False
                )

    def to_dataframe(
        self,
        table_name: str = None,
        filter_clause: str = None,
        list_columns: list = None,
    ):
        if not table_name:
            raise ValueError("table_name and list_columns are required")
        columns = ""
        df = None
        query = ""

        if list_columns:
            columns = ", ".join([column for column in list_columns])
        else:
            columns = "*"

        if filter_clause:
            query = f"SELECT {columns} FROM {table_name} WHERE {filter_clause}"
        else:
            query = f"SELECT {columns} FROM {table_name}"

        with self.db.session_scope() as session:
            df = pd.read_sql(query, con=session.bind)
        return df



from unittest.mock import patch
from dhuolib.clients.platform import DhuolibPlatformClient
import unittest
import sys

sys.path.append("src")


class TestDhuolibPlatform(unittest.TestCase):
    def setUp(self):
        self.endpoint = "https://dhuo-data-api-data-service-stg.br.engineering"
        self.dhuolib = DhuolibPlatformClient(
            service_uri=self.endpoint,
            project_name="ex",
            token="Bearer eyJ0eXAiOi"
        )

    @patch("requests.post")
    def test_0_create_batch_project_successful(self, mock_post):
        project_name = "teste-project"
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {"status": "success", "project_id": 123}
        response = self.dhuolib.create_batch_project(project_name)
        self.assertEqual(response, {"status": "success", "project_id": 123})

    def test_1_create_batch_project_raises_exception_on_none(self):
        with self.assertRaises(ValueError) as context:
            self.dhuolib.create_batch_project(project_name="teste_%$%erro")
            self.assertIn("project_name is required", str(context.exception))

    def test_2_deploy_batch_project_no_project_name(self):
        self.dhuolib = DhuolibPlatformClient(service_uri=self.endpoint, token="123")
        with self.assertRaises(ValueError) as context:
            self.dhuolib.deploy_batch_project(
                script_filename="script.py", requirements_filename="requirements.txt"
            )
            self.assertIn("Batch project is required", str(context.exception))

    def test_3_deploy_batch_project_missing_files(self):
        with self.assertRaises(ValueError) as context:
            self.dhuolib.deploy_batch_project(None, None)
            self.assertIn(
                "script_filename and requirements_filename are required",
                str(context.exception),
            )

    def test_4_deploy_batch_project_file_not_found(self):
        with patch("builtins.open", side_effect=FileNotFoundError("File not found")):
            response = self.dhuolib.deploy_batch_project(
                script_filename="scrt.py", requirements_filename="requirements.txt"
            )
            self.assertEqual(response, {"error": "File not found"})

    def test_5_schedule_batch_run_no_project_name(self):
        self.dhuolib = DhuolibPlatformClient(service_uri=self.endpoint, token="123")
        with self.assertRaises(ValueError) as context:
            self.dhuolib.schedule_batch_run(None, "1h")
            self.assertIn("Batch project is required", str(context.exception))

    def test_6_remove_schedule_no_project_name(self):
        self.dhuolib = DhuolibPlatformClient(service_uri=self.endpoint, token="123")
        with self.assertRaises(ValueError) as context:
            self.dhuolib.remove_schedule(None)
            self.assertIn("Batch project is required", str(context.exception))

    def test_7_cluster_with_no_project_name(self):
        self.dhuolib = DhuolibPlatformClient(service_uri=self.endpoint, token="123")
        with self.assertRaises(ValueError) as context:
            self.dhuolib.create_cluster()
            self.assertIn("Batch project is required", str(context.exception))



import sys
import unittest
from unittest.mock import patch

sys.path.append("src")

from dhuolib.clients.experiment import DhuolibExperimentMLClient


class TestDhuolibPlatformClient(unittest.TestCase):
    def setUp(self):
        self.end_point = "http://localhost:8000"
        self.dhuolib = DhuolibExperimentMLClient(
            service_uri=self.end_point, token="123"
        )

    def test_0_invalid_run_params(self):
        response = self.dhuolib.create_experiment(experiment_name="teste1")
        self.assertEqual(list(response.keys()), ["error"])

    @patch("requests.post")
    def test_1_create_experiment_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {"experiment_id": "1"}

        response = self.dhuolib.create_experiment(
            experiment_name="teste1",
            experiment_tags={"version": "v1", "priority": "P1"},
        )
        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_3_predict_online_with_valid_dataset(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "model_name": "nlp_framework",
            "predictions": [0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
        }

        run_params = {
            "stage": "Production",
            "data": "tests/files/data_predict.csv",
            "modelname": "nlp_framework",
        }

        response = self.dhuolib.predict_online(run_params)

        self.assertEqual(response, mock_response.json.return_value)



from sqlalchemy import text
from dhuolib.repository import DatabaseConnection, GenericRepository
import unittest
import sys
import pandas as pd

sys.path.append("src")


class TestDatabaseConnection(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config_file_name = "tests/files/database.json"
        cls.db_connection = DatabaseConnection(config_file_name=cls.config_file_name)

    def test_database_connection_init(self):
        db_connection = DatabaseConnection(self.config_file_name)
        self.assertIsNotNone(db_connection.engine)
        self.assertIsNotNone(db_connection.session)

    def test_session_scope(self):
        with self.db_connection.session_scope() as session:
            self.assertIsNotNone(session)


class TestGenericRepository(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config_file_name = "tests/files/database.json"
        cls.db_connection = DatabaseConnection(config_file_name=cls.config_file_name)
        cls.table_name = "test_table"
        cls.repo = GenericRepository(db_connection=cls.db_connection)
        cls.repo.db.session.execute(
            text(
                f"CREATE TABLE IF NOT EXISTS {cls.table_name} (id INTEGER PRIMARY KEY, name VARCHAR(255), age INTEGER, predict VARCHAR(255), version VARCHAR(255), created_at TIMESTAMP)"
            )
        )

    @classmethod
    def tearDownClass(cls):
        with cls.db_connection.session_scope() as session:
            session.execute(text(f"DROP TABLE IF EXISTS {cls.table_name}"))

    def setUp(self):
        self.db_connection = self.__class__.db_connection
        self.repo = self.__class__.repo
        self.table_name = self.__class__.table_name

    def tearDown(self):
        with self.db_connection.session_scope() as session:
            session.execute(text(f"DELETE FROM {self.table_name}"))

    def test_1_update_table_by_dataframe(self):
        initial_data = pd.DataFrame(
            [{"id": 1, "name": "John Doe", "age": 30, "predict": "Positive"}]
        )
        self.repo.update_table_by_dataframe(
            table_name=self.table_name,
            dataframe=initial_data,
            if_exists="append",
            is_update_version=True,
        )

        new_data = pd.DataFrame(
            [
                {
                    "id": 2,
                    "name": "Jane Doe",
                    "age": 25,
                    "predict": "Positive",
                    "version": 1,
                    "created_at": "2021-01-01",
                },
                {
                    "id": 3,
                    "name": "Jim Beam",
                    "age": 35,
                    "predict": "Negative",
                    "version": 1,
                    "created_at": "2021-01-01",
                },
            ]
        )
        self.repo.update_table_by_dataframe(self.table_name, new_data)

        df = self.repo.to_dataframe(self.table_name)
        self.assertEqual(len(df), 3)
        self.assertListEqual(list(df["name"]), ["John Doe", "Jane Doe", "Jim Beam"])
        self.assertListEqual(list(df["predict"]), ["Positive", "Positive", "Negative"])

    def test_2_create_table_by_dataframe(self):
        with self.db_connection.session_scope() as session:
            session.execute(text(f"DROP TABLE IF EXISTS {self.table_name}"))
        data = pd.DataFrame(
            [
                {
                    "id": 1,
                    "name": "John Doe",
                    "age": 30,
                    "predict": "Positive",
                    "score": 0.932,
                },
                {
                    "id": 2,
                    "name": "Jane Smith",
                    "age": 25,
                    "predict": "Negative",
                    "score": 0.1332,
                },
                {
                    "id": 3,
                    "name": "Bob Johnson",
                    "age": 35,
                    "predict": "Positive",
                    "score": 0.53,
                },
            ]
        )
        self.repo.create_table_by_dataframe(self.table_name, data)
        with self.repo.db.session_scope() as session:
            result = session.execute(
                text(f"SELECT * FROM {self.table_name}")
            ).fetchall()
            self.assertEqual(len(result), 3)
            self.assertEqual(result[0].name, "John Doe")
            self.assertEqual(result[1].name, "Jane Smith")
            self.assertEqual(result[2].name, "Bob Johnson")

    def test_3_update_table_by_dataframe_with_versioning(self):
        with self.repo.db.session_scope() as session:
            session.execute(text(f"DROP TABLE {self.table_name}"))
        data = pd.DataFrame(
            [
                {
                    "id": 1,
                    "name": "John Doe",
                    "age": 30,
                    "predict": "",
                    "created_at": "2021-01-01",
                    "version": 1,
                },
                {
                    "id": 2,
                    "name": "Jane Smith",
                    "age": 25,
                    "predict": "",
                    "created_at": "2021-01-01",
                    "version": 1,
                },
                {
                    "id": 3,
                    "name": "Bob Johnson",
                    "age": 35,
                    "created_at": "2021-01-01",
                    "version": 1,
                    "predict": "",
                },
            ]
        )
        response = self.repo.create_table_by_dataframe(self.table_name, data)
        self.assertEqual(response["number_lines_inserted"], 3)

        update_data = pd.DataFrame(
            [
                {"id": 1, "name": "John Doe", "age": 30, "predict": "Positive"},
                {"id": 2, "name": "Jane Smith", "age": 25, "predict": "Positive"},
                {"id": 3, "name": "Bob Johnson", "age": 35, "predict": "Negative"},
            ]
        )

        self.repo.update_table_by_dataframe(
            table_name=self.table_name,
            dataframe=update_data,
            if_exists="append",
            is_update_version=True,
        )

        df = self.repo.to_dataframe(self.table_name)
        self.assertListEqual(
            list(df["name"]),
            [
                "John Doe",
                "Jane Smith",
                "Bob Johnson",
                "John Doe",
                "Jane Smith",
                "Bob Johnson",
            ],
        )
        self.assertListEqual(
            list(df["predict"]), ["", "", "", "Positive", "Positive", "Negative"]
        )
        self.assertListEqual(list(df["version"]), [1, 1, 1, 2, 2, 2])



import pandas as pd
import json
import requests
from pydantic import ValidationError
from werkzeug.datastructures import FileStorage

import dhuolib.utils as ut
from dhuolib.config import logger
from dhuolib.services import ServiceAPIMLFacade
from dhuolib.utils import validate_name
from dhuolib.validations import (
    ExperimentBody,
    PredictBatchModelBody,
    PredictOnlineModelBody,
    RunExperiment,
    TransitionModel,
    RunSearch,
    ModelBody,
)


class DhuolibExperimentMLClient:
    def __init__(self, service_uri: str, token: str = None):
        if not service_uri:
            raise ValueError("service_uri is required")

        if not token:
            raise ValueError("token is required")

        self.service = ServiceAPIMLFacade(service_uri=service_uri, token=token)

    def create_experiment(
        self, experiment_name: str, experiment_tags: dict = None
    ) -> dict:
        try:
            experiment = ExperimentBody.model_validate(
                {
                    "experiment_name": validate_name(experiment_name),
                    "experiment_tags": experiment_tags,
                }
            )
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        try:
            response = self.service.experiment_api.create(experiment.model_dump())
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: Failed to create experiment: {e}")
            return {"error": str(e)}

        logger.info(f"Experiment Name: {experiment.experiment_name} created")

        return response

    def predict_online(self, run_params) -> dict:
        try:
            PredictOnlineModelBody.model_validate(run_params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        try:
            with open(run_params["data"], "rb") as f1:
                files = {
                    "data": FileStorage(
                        stream=f1, filename="data.csv", content_type="csv"
                    )
                }

                try:
                    response = self.service.experiment_api.predict_online(
                        params=run_params, files=files
                    )
                except requests.exceptions.HTTPError as e:
                    logger.error(f"Error: {e}")
                    return {"error": str(e)}
                logger.info(f"Model Name: {run_params['modelname']} predictions")
                return response
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def _get_model_to_prediction(self, predict: PredictBatchModelBody) -> dict:
        try:
            path_to_pickle = self.service.experiment_api.download_pickle(
                model_name=predict.modelname,
                model_stage=predict.stage,
                run_id=predict.run_id,
                experiment_name=predict.experiment_name,
                local_filename=predict.batch_model_dir,
                type_model=predict.type_model,
            )
        except requests.exceptions.HTTPError as e:
            logger.error(
                f"Error: verify the model name, experiment_name, stage and run_id: {e}"
            )
            return {"error": str(e)}

        return ut.load_pickle_model(path_to_pickle)

    def prediction_batch_with_dataframe(
        self, batch_params: dict, df: pd.DataFrame
    ) -> dict:
        try:
            predict = PredictBatchModelBody.model_validate(batch_params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

        if not predict.run_id and not predict.stage:
            msg_err = "run_id or stage are required"
            logger.error(msg_err)
            raise ValueError(msg_err)

        model = self._get_model_to_prediction(predict)
        if isinstance(model, dict):
            return model
        return model.predict(df)

    def download_pkl(self, batch_params: dict):
        try:
            predict = PredictBatchModelBody.model_validate(batch_params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

        if not predict.run_id and not predict.stage:
            msg_err = "run_id or stage are required"
            logger.error(msg_err)
            raise ValueError(msg_err)

        model = self._get_model_to_prediction(predict)
        return model

    def search_experiments(
        self, filter_string: str = "", max_results: int = 10, page_token: str = ""
    ) -> dict:
        try:
            response = self.service.experiment_api.search(
                filter_string=filter_string,
                max_results=max_results,
                page_token=page_token,
            )
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

        return response

    def execute_run_for_experiment(
        self,
        type_model: str,
        experiment_name: str,
        modelpkl_path: str,
        requirements_path: str,
        tags={},
    ) -> dict:

        if not experiment_name or not modelpkl_path or not requirements_path:
            raise ValueError(
                "experiment_name, modelpkl_path and requirements_path are required"
            )
        params = {
            "type_model": type_model,
            "experiment_name": experiment_name,
            "tags": tags,
        }

        try:
            RunExperiment.model_validate(params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

        try:
            with open(requirements_path, "rb") as f1, open(modelpkl_path, "rb") as f2:
                files = {
                    "requirements_path": (requirements_path, f1, "text/plain"),
                    "modelpkl_path": (modelpkl_path, f2, "application/octet-stream"),
                }

                chunk_size = 10 * 1024 * 1024
                for file_name, file_info in files.items():
                    file_path, file_obj, content_type = file_info
                    while True:
                        chunk = file_obj.read(chunk_size)
                        if not chunk:
                            break
                        try:
                            self.service.runs_api.upload_chunk(
                                chunk=chunk,
                                content_type=content_type,
                                experiment_name=experiment_name,
                                file_name=file_name,
                                file_path=file_path,
                            )
                        except requests.exceptions.HTTPError as e:
                            logger.error(f"Error: {e}")
                            return {"error": str(e)}
                try:
                    response = self.service.runs_api.create(run_params=params)
                except requests.exceptions.HTTPError as e:
                    logger.error(f"Error: check the passed parameters: {e}")
                    return {"error": f"Error: check the passed parameters: {e}"}

                logger.info(f"Experiment Name: {params['experiment_name']} running")
                return response
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def search_runs(
        self,
        filter_string: str = "",
        max_results: int = 10,
        page_token: str = "",
        experiment_name: str = "",
    ) -> dict:

        if experiment_name == "":
            raise ValueError("experiment_name is required")
        try:
            RunSearch.model_validate(
                {
                    "experiment_name": experiment_name,
                    "filter_string": filter_string,
                    "max_results": max_results,
                    "page_token": page_token,
                }
            )
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        try:
            response = self.service.runs_api.search(
                filter_string=filter_string,
                max_results=max_results,
                page_token=page_token,
                experiment_name=experiment_name,
            )
            return response
        except ValueError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def create_model(
        self, modelname: str, stage: str, run_id: str, model_uri: str, tags: dict
    ) -> dict:
        try:
            transition_model = ModelBody.model_validate(
                {
                    "modelname": validate_name(modelname),
                    "stage": stage,
                    "run_id": run_id,
                    "model_uri": model_uri,
                    "tags": tags,
                }
            )
            response = self.service.model_api.create(transition_model.model_dump())
            return response
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: check the passed parameters: {e}")
            return {"error": f"Error: check the passed parameters: {e}"}

    def transition_model(self, model_name: str, version: str, stage: str) -> dict:
        try:
            TransitionModel.model_validate(
                {"model_name": model_name, "version": version, "stage": stage}
            )
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

        try:
            response = self.service.model_api.transition_model(
                model_name=model_name, version=version, stage=stage
            )
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: check the passed parameters: {e}")
            return {"error": f"Error: check the passed parameters: {e}"}

        return response

    def search_models(
        self, filter_string: str = "", max_results: int = 10, page_token: str = ""
    ) -> dict:
        try:
            response = self.service.model_api.search(
                filter_string=filter_string,
                max_results=max_results,
                page_token=page_token,
            )
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        return response



import pickle


def validate_name(string: str):
    if string is None:
        raise ValueError("string must be informed!")
    if len(string) < 5:
        raise ValueError("string must have at least 5 characters!")
    if len(string) > 30:
        raise ValueError("string cannot be longer than 30 characters!")
    # special characters not allowed
    special_characters = "\"'!@#$%¨&*()+=§`[]{}ªº;.,~^/?\\|áéíóúâêîôûãõäëïöüñç "
    string = string.lower()
    if any(c in special_characters for c in string):
        raise ValueError("string has invalid characters. Only '-' and '_' are allowed!")
    # start and end
    if string.startswith("_"):
        raise ValueError("string can't start with _!")
    if string.startswith("-"):
        raise ValueError("string can't start with -!")
    if string.endswith("_"):
        raise ValueError("string can't end with _!")
    if string.endswith("-"):
        raise ValueError("string can't end with -!")

    return string


def load_pickle_model(pickle_path):
    with open(pickle_path, "rb") as f:
        model = pickle.load(f)
    return model



import sys
import unittest
from unittest.mock import patch

sys.path.append("src")


from dhuolib.clients.experiment import DhuolibExperimentMLClient


class TestDhuolibPlatformClient(unittest.TestCase):
    def setUp(self):
        self.end_point = "http://localhost:8000"
        self.dhuolib = DhuolibExperimentMLClient(
            service_uri=self.end_point, token="123"
        )

    @patch("requests.post")
    def test_1_create_model_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "current_stage": "Production",
            "last_updated_timestamp": 1713582060414,
            "model_version": "1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "status": "READY",
        }

        response = self.dhuolib.create_model(
            modelname="nlp_framework",
            stage="Production",
            run_id="9434e517ed104958b6f5f47d33c79184",
            model_uri="model_uri",
            tags={"version": "v1", "priority": "P1"},
        )

        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.get")
    def test_2_transition_model_with_valid_params(self, mock_get):
        model_name = "nlp_framework"
        version = "117"
        stage = "Production"
        expected_response = {
            "current_stage": "Production",
            "last_updated_timestamp": 1713582060414,
            "model_version": "1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "status": "READY",
        }

        mock_get.return_value.status_code = 200
        mock_get.return_value.json.return_value = expected_response

        response = self.dhuolib.transition_model(model_name, version, stage)
        self.assertEqual(response, expected_response)

    def test_3_transition_model_with_invalid_params(self):
        with self.assertRaises(TypeError):
            self.dhuolib.transition_model(model_name="nlp_framework")
        with self.assertRaises(TypeError):
            self.dhuolib.transition_model(stage="Production")
        with self.assertRaises(TypeError):
            self.dhuolib.transition_model(version="117")
